{% if user_was_incorrect %}

	<div class="alert alert-error">

		<strong>You answered the above question incorrectly</strong>

	</div>


{% endif %}

<table class="table table-striped table-bordered">
  <tbody>

	{% for answer in answers %}



      {% if answer.correct %}

	    <tr class="success">
		  <td>{{ answer.content }}</td>
		  <td><strong>This is the correct answer</strong></td>


      {% else %}
		<tr>
    	  <td>{{ answer.content }}</td>
		  <td>
			{% ifequal question_type 'MCQuestion' %}
			  {% if answer.id == previous_answer_id %}
			    This was your answer.
			  {% endif %}
			{% endifequal %}
		  </td>

	  {% endif %}
	    </tr>

	{% endfor %}

  </tbody>
</table>
