<!DOCTYPE html>
<html xmlns:fb="http://www.facebook.com/2008/fbml" xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en" dir="ltr">
<head>
<meta charset="utf-8">

<title>Example Quiz Website | {% block title %}{% endblock %}</title>
<meta name="description" content="{% block description %}{% endblock %}">

<!-- styles -->
<link rel="stylesheet" href="http://netdna.bootstrapcdn.com/bootstrap/3.1.1/css/bootstrap.min.css">

<!-- HTML5 shim, for IE6-8 support of HTML5 elements -->
<!--[if lt IE 9]>
<script src="//html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->

</head>

<body>


<!-- Before the quiz content -->

{% block content %}

{% endblock %}

<!-- After the quiz content -->


</body>
</html>
