<tr>
  <td>
	<label>
	  <input type="radio" name="guess" value="T" class="form-radio">
	  True
	</label>
  </td>
</tr>
<tr>
  <td>
	<label>
	  <input type="radio" name="guess" value="F" class="form-radio">
	  False
	</label>
  </td>
</tr>
