{% extends 'base.html' %}
{% block title %}All Quizzes{% endblock %}

{% block content %}
<h2>Quiz list</h2>
    {% if quiz_list %}
        <ul>
        {% for quiz in quiz_list %}
            <li><a href="{% url 'quiz.views.quiz_take' quiz_name=quiz.url %}">{{ quiz.title }}</a></li>
        {% endfor %}
        </ul>
    {% else %}
        <p>There are no available quizzes.</p>
    {% endif %}
{% endblock %}

{% block right-sidebar %}
<h2>Sponsored Advertisements</h2>
{% endblock %}
